@@ -1,12 +1,13 @@
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT License
 import abc
+import time
 import webbrowser
 from threading import Lock
 from typing import Callable, Optional
 
 from azure.core.exceptions import ClientAuthenticationError
-from azure.identity import ManagedIdentityCredential
+from azure.identity import ManagedIdentityCredential, AzureCliCredential
 from msal import ConfidentialClientApplication, PublicClientApplication
 
 from ._cloud_settings import CloudSettings
@@ -21,7 +22,7 @@ except ImportError:
 
 
 try:
-    from azure.identity.aio import ManagedIdentityCredential as AsyncManagedIdentityCredential
+    from azure.identity.aio import ManagedIdentityCredential as AsyncManagedIdentityCredential, AzureCliCredential as AsyncAzureCliCredential
 except ImportError:
 
     class AsyncManagedIdentityCredential:
@@ -44,10 +45,6 @@ class TokenConstants:
     MSAL_INTERACTIVE_PROMPT = "select_account"
     AZ_TOKEN_TYPE = "tokenType"
     AZ_ACCESS_TOKEN = "accessToken"
-    AZ_REFRESH_TOKEN = "refreshToken"
-    AZ_USER_ID = "userId"
-    AZ_AUTHORITY = "_authority"
-    AZ_CLIENT_ID = "_clientId"
 
 
 class TokenProviderBase(abc.ABC):
@@ -69,7 +66,7 @@ class TokenProviderBase(abc.ABC):
             self._cloud_info = CloudSettings.get_cloud_info()
 
     def get_token(self):
-        """ Get a token silently from cache or authenticate if cached token is not found """
+        """Get a token silently from cache or authenticate if cached token is not found"""
 
         if not self._initialized:
             with TokenProviderBase.lock:
@@ -85,13 +82,13 @@ class TokenProviderBase(abc.ABC):
         return self._valid_token_or_throw(token)
 
     async def get_token_async(self):
-        """ Get a token asynchronously silently from cache or authenticate if cached token is not found """
+        """Get a token asynchronously silently from cache or authenticate if cached token is not found"""
         with TokenProviderBase.lock:
             if not self._initialized:
                 self._init_impl()
                 self._initialized = True
 
-        token = await self._get_token_from_cache_impl_async()
+        token = self._get_token_from_cache_impl()
 
         if token is None:
             with TokenProviderBase.lock:
@@ -102,37 +99,33 @@ class TokenProviderBase(abc.ABC):
     @staticmethod
     @abc.abstractmethod
     def name() -> str:
-        """ return the provider class name """
+        """return the provider class name"""
         pass
 
     @abc.abstractmethod
     def context(self) -> dict:
-        """ return a secret-free context for error reporting """
+        """return a secret-free context for error reporting"""
         pass
 
     @abc.abstractmethod
     def _init_impl(self):
-        """ Implement any "heavy" first time initializations here """
+        """Implement any "heavy" first time initializations here"""
         pass
 
     @abc.abstractmethod
     def _get_token_impl(self) -> Optional[dict]:
-        """ implement actual token acquisition here """
+        """implement actual token acquisition here"""
         pass
 
+    async def _get_token_impl_async(self) -> Optional[dict]:
+        """implement actual token acquisition here"""
+        return await sync_to_async(self._get_token_impl)()
+
     @abc.abstractmethod
     def _get_token_from_cache_impl(self) -> Optional[dict]:
-        """ Implement cache checks here, return None if cache check fails """
+        """Implement cache checks here, return None if cache check fails"""
         pass
 
-    async def _get_token_from_cache_impl_async(self) -> Optional[dict]:
-        """ Implement cache checks here, return None if cache check fails """
-        return await sync_to_async(self._get_token_from_cache_impl)()
-
-    async def _get_token_impl_async(self) -> Optional[dict]:
-        """ implement actual token acquisition here """
-        return await sync_to_async(self._get_token_impl)()
-
     @staticmethod
     def _valid_token_or_none(token: dict) -> Optional[dict]:
         if token is None or TokenConstants.MSAL_ERROR in token:
@@ -154,7 +147,7 @@ class TokenProviderBase(abc.ABC):
 
 
 class BasicTokenProvider(TokenProviderBase):
-    """ Basic Token Provider keeps and returns a token received on construction """
+    """Basic Token Provider keeps and returns a token received on construction"""
 
     def __init__(self, token: str):
         super().__init__(None)
@@ -178,7 +171,7 @@ class BasicTokenProvider(TokenProviderBase):
 
 
 class CallbackTokenProvider(TokenProviderBase):
-    """ Callback Token Provider generates a token based on a callback function provided by the caller """
+    """Callback Token Provider generates a token based on a callback function provided by the caller"""
 
     def __init__(self, token_callback: Callable[[], str]):
         super().__init__(None)
@@ -256,19 +249,15 @@ class MsiTokenProvider(TokenProviderBase):
     def _get_token_from_cache_impl(self) -> dict:
         return None
 
-    async def _get_token_from_cache_impl_async(self) -> Optional[dict]:
-        return None
-
 
 class AzCliTokenProvider(TokenProviderBase):
-    """ AzCli Token Provider obtains a refresh token from the AzCli cache and uses it to authenticate with MSAL """
+    """AzCli Token Provider obtains a refresh token from the AzCli cache and uses it to authenticate with MSAL"""
 
     def __init__(self, kusto_uri: str):
         super().__init__(kusto_uri)
-        self._msal_client = None
-        self._client_id = None
-        self._authority_uri = None
-        self._username = None
+        self._az_auth_context = None
+        self._az_auth_context_async = None
+        self._az_token = None
 
     @staticmethod
     def name() -> str:
@@ -281,81 +270,41 @@ class AzCliTokenProvider(TokenProviderBase):
         pass
 
     def _get_token_impl(self) -> dict:
-        # try and obtain the refresh token from AzCli
-        refresh_token = None
-        token = None
-        stored_token = self._get_azure_cli_auth_token()
-        if TokenConstants.AZ_REFRESH_TOKEN in stored_token and TokenConstants.AZ_CLIENT_ID in stored_token and TokenConstants.AZ_AUTHORITY in stored_token:
-            refresh_token = stored_token[TokenConstants.AZ_REFRESH_TOKEN]
-            self._client_id = stored_token[TokenConstants.AZ_CLIENT_ID]
-            self._authority_uri = stored_token[TokenConstants.AZ_AUTHORITY]
-            self._username = stored_token[TokenConstants.AZ_USER_ID]
-        else:
-            raise KustoClientError("Unable to obtain a refresh token from Az-Cli. Calling 'az login' may fix this issue.")
-
-        if self._msal_client is None:
-            self._msal_client = PublicClientApplication(client_id=self._client_id, authority=self._authority_uri)
-
         try:
-            token = self._msal_client.acquire_token_by_refresh_token(refresh_token, self._scopes)
-        except Exception as ex:
-            raise KustoClientError("Unable to obtain with Az-Cli refresh token. Calling 'az login' may fix this issue.\n" + str(ex))
+            if self._az_auth_context is None:
+                self._az_auth_context = AzureCliCredential()
 
-        return self._valid_token_or_throw(token, "Calling 'az login' may fix this issue.")
+            self._az_token = self._az_auth_context.get_token(self._kusto_uri)
+            return {TokenConstants.AZ_TOKEN_TYPE: TokenConstants.BEARER_TYPE, TokenConstants.AZ_ACCESS_TOKEN: self._az_token.token}
+        except Exception as e:
+            raise KustoClientError(
+                "Failed to obtain Az Cli token for '{0}'.\nPlease be sure AzCli version 2.3.0 and above is intalled.\n{1}".format(self._kusto_uri, e)
+            )
+
+    async def _get_token_impl_async(self) -> Optional[dict]:
+        try:
+            if self._az_auth_context_async is None:
+                self._az_auth_context_async = AsyncAzureCliCredential()
+
+            self._az_token = await self._az_auth_context_async.get_token(self._kusto_uri)
+            return {TokenConstants.AZ_TOKEN_TYPE: TokenConstants.BEARER_TYPE, TokenConstants.AZ_ACCESS_TOKEN: self._az_token.token}
+        except Exception as e:
+            raise KustoClientError(
+                "Failed to obtain Az Cli token for '{0}'.\nPlease be sure AzCli version 2.3.0 and above is installed.\n{1}".format(self._kusto_uri, e)
+            )
 
     def _get_token_from_cache_impl(self) -> dict:
-        token = None
-        if self._msal_client is not None:
-            account = None
-            if self._username is not None:
-                accounts = self._msal_client.get_accounts(self._username)
-                if len(accounts) > 0:
-                    account = accounts[0]
+        if self._az_token is not None:
+            # A token is considered valid if it is due to expire in no less than 10 minutes
+            cur_time = time.time()
+            if (self._az_token.expires_on - 600) > cur_time:
+                return {TokenConstants.MSAL_TOKEN_TYPE: TokenConstants.BEARER_TYPE, TokenConstants.MSAL_ACCESS_TOKEN: self._az_token.token}
 
-            token = self._msal_client.acquire_token_silent(scopes=self._scopes, account=account, client_id=self._client_id, authority=self._authority_uri)
-
-        return self._valid_token_or_none(token)
-
-    @staticmethod
-    def _get_azure_cli_auth_token() -> dict:
-        """
-        Try to get the az cli authenticated token
-        :return: refresh token
-        """
-        import os
-
-        try:
-            # this makes it cleaner, but in case azure cli is not present on virtual env,
-            # but cli exists on computer, we can try and manually get the token from the cache
-            from azure.cli.core._profile import Profile
-            from azure.cli.core._session import ACCOUNT
-            from azure.cli.core._environment import get_config_dir
-
-            azure_folder = get_config_dir()
-            ACCOUNT.load(os.path.join(azure_folder, "azureProfile.json"))
-            profile = Profile(storage=ACCOUNT)
-            token_data = profile.get_raw_token()[0][2]
-
-            return token_data
-
-        except ModuleNotFoundError:
-            try:
-                import os
-                import json
-
-                folder = os.getenv("AZURE_CONFIG_DIR", None) or os.path.expanduser(os.path.join("~", ".azure"))
-                token_path = os.path.join(folder, "accessTokens.json")
-                with open(token_path) as f:
-                    data = json.load(f)
-
-                # TODO: not sure I should take the first
-                return data[0]
-            except Exception as e:
-                raise KustoClientError("Azure cli token was not found. Please run 'az login' to setup account.", e)
+        return None
 
 
 class UserPassTokenProvider(TokenProviderBase):
-    """ Acquire a token from MSAL with username and password """
+    """Acquire a token from MSAL with username and password"""
 
     def __init__(self, kusto_uri: str, authority_uri: str, username: str, password: str):
         super().__init__(kusto_uri)
@@ -390,7 +339,7 @@ class UserPassTokenProvider(TokenProviderBase):
 
 
 class DeviceLoginTokenProvider(TokenProviderBase):
-    """ Acquire a token from MSAL with Device Login flow """
+    """Acquire a token from MSAL with Device Login flow"""
 
     def __init__(self, kusto_uri: str, authority_uri: str, device_code_callback=None):
         super().__init__(kusto_uri)
@@ -437,7 +386,7 @@ class DeviceLoginTokenProvider(TokenProviderBase):
 
 
 class InteractiveLoginTokenProvider(TokenProviderBase):
-    """ Acquire a token from MSAL with Device Login flow """
+    """Acquire a token from MSAL with Device Login flow"""
 
     def __init__(self, kusto_uri: str, authority_uri: str, login_hint: Optional[str] = None, domain_hint: Optional[str] = None):
         super().__init__(kusto_uri)
@@ -474,7 +423,7 @@ class InteractiveLoginTokenProvider(TokenProviderBase):
 
 
 class ApplicationKeyTokenProvider(TokenProviderBase):
-    """ Acquire a token from MSAL with application Id and Key """
+    """Acquire a token from MSAL with application Id and Key"""
 
     def __init__(self, kusto_uri: str, authority_uri: str, app_client_id: str, app_key: str):
         super().__init__(kusto_uri)
